/*----------------------------------------------------------------------- 
 * 
 * File Name: ConditionData.c
 *
 * Author: Brady, P. R. and Brown, D. A.
 * 
 * Revision: $Id: ConditionData.c,v 1.27 2001/08/27 17:23:23 patrick Exp $
 * 
 *-----------------------------------------------------------------------
 */

#include "ConditionData.h"

/*
 *
 * prototypes for debugging functions
 *
 */


#ifdef INSPIRAL_SO_DEBUG
static
void
PrintInspiralBank (
    LALStatus           *status,
    InspiralTemplate    *head,
    FILE                *fp
    );

static
void
PrintTemplateNodeList(
    LALStatus                  *status,
    InspiralTemplateNode       *tmpltHead,
    FILE                       *fp
    );
#endif


NRCSID( CONDITIONDATAC, PKG "$Id: ConditionData.c,v 1.27 2001/08/27 17:23:23 patrick Exp $" );


void
LALConditionData(
    LALStatus             *status,
    LALSearchInput        *inout,
    void                  *searchParams,
    LALMPIParams          *mpiParams
                )
{
  UINT4                         i, j;
  UINT4                         oneSidedPSD = 0;
  UINT4                         dataIndex   = 0;
  UINT4                         specIndex   = 0;
  UINT4                         respIndex   = 0;
  INT8                          dataTimeNS  = 0;
  REAL4                        *dummyData   = NULL;
  REAL4                        *dummySpec   = NULL;
  COMPLEX8                     *dummyResp   = NULL;
  multiDimData                 *mddTmplt    = NULL;
  DataSegmentVector            *dataSegVec  = NULL;
  InspiralSearchParams         *params      = NULL;
  InspiralTemplate             *bankCurrent = NULL;
  InspiralTemplateNode         *tmpltNode   = NULL;

  INITSTATUS( status, "LALConditionData", CONDITIONDATAC );
  ATTATCHSTATUSPTR( status );


  /* 
   *
   * check and cast parameter structure
   *
   */


  ASSERT( searchParams, status,
      CONDITIONDATAH_ENULL, CONDITIONDATAH_MSGENULL );

  params = (InspiralSearchParams *) searchParams;


  /* 
   *
   * identify multiDimData sequences by their names 
   *
   */


  ASSERT (inout, status, 
      CONDITIONDATAH_ENULL, CONDITIONDATAH_MSGENULL);

  for ( i = 0; i < inout->numberSequences; ++i )
  {
    if ( strstr( inout->sequences[i].name, INPUTNAME_CHANNEL ) )
    {
      dataIndex = i;
      dummyData   = inout->sequences[i].data.real4;
      dataTimeNS  = 1000000000L * 
            (INT8) inout->sequences[i].range.dTime.startSec;
      dataTimeNS += (INT8) inout->sequences[i].range.dTime.startNan;
    }
    else if ( strstr( inout->sequences[i].name, INPUTNAME_SPECTRUM ) )
    {
      specIndex = i;
      dummySpec   = inout->sequences[i].data.real4;
    }
    else if ( strstr( inout->sequences[i].name, INPUTNAME_RESPONSE ) )
    {
      respIndex = i;
      dummyResp   = inout->sequences[i].data.complex8;
    }
    else if ( strstr( inout->sequences[i].name, INPUTNAME_TMPLTBANK ) )
    {
      mddTmplt = &( inout->sequences[i] );
    }
    else
    {
      ABORT( status, CONDITIONDATAH_EINPT, CONDITIONDATAH_MSGEINPT ); 
    }
  }



  if ( params->searchMaster ) /* search master */
  {


    /*
     *
     * get templates from ilwd if necessary and check that a bank exists
     *
     */


    /* should we look for templates in the ilwd? */
    if ( params->tmpltSource == 2 )
    {
      InspiralTmpltsFromILWD( status->statusPtr, params, mddTmplt );
      CHECKSTATUSPTR( status );
    }

    /* abort if we don't have a bank by now */
    if ( ! params->bankHead )
    {
      ABORT( status, CONDITIONDATAH_ENBNK, CONDITIONDATAH_MSGENBNK );
    }

#ifdef INSPIRAL_SO_DEBUG
      /* print out the template bank */
      PrintInspiralBank( status->statusPtr, params->bankHead, stdout ); 
      CHECKSTATUSPTR( status );
#endif


      /*
       *
       * create the linked list of template nodes for coarse templates
       *
       */


      /* make sure the tmpltHead is NULL before we start */
      if ( params->tmpltHead )
      {
        ABORT( status, CONDITIONDATAH_ENNUL, CONDITIONDATAH_MSGENNUL );
      }

      /* we need to keep track of the number of templates for progress */
      params->numTmplts = 0;
      
      /* create the template node list */
      for ( bankCurrent = params->bankHead; bankCurrent; 
          bankCurrent = bankCurrent->next, params->numTmplts++ )
      {
        LALFindChirpCreateTmpltNode( status->statusPtr, 
            bankCurrent, &tmpltNode );
        CHECKSTATUSPTR( status );

        /* make sure we filter every segment against every coarse tmplt */
        memset( tmpltNode->segmentIdVec->data, 0,
            tmpltNode->segmentIdVec->length * sizeof(INT4) );
        for ( i = 0; i < tmpltNode->segmentIdVec->length; i++ )
        {
          tmpltNode->segmentIdVec->data[i] = 1;
        }

        if ( ! params->tmpltHead ) params->tmpltHead = tmpltNode;
      }

      /* set the current template node to the head of the bank */
      params->tmpltCurrent = params->tmpltHead;

#ifdef INSPIRAL_SO_DEBUG
      /* print out the template node list */
      PrintTemplateNodeList( status->statusPtr, params->tmpltHead, stdout );
      CHECKSTATUSPTR( status );
#endif

  }
  else /* search slave */
  {


    /*
     *
     * allocate memory for data segments
     *
     */


    LALCreateDataSegmentVector( status->statusPtr, &dataSegVec, 
        params->initParams );
    CHECKSTATUSPTR( status );


    /* 
     *
     * translate InPut to DataSegment 
     *
     */


    for ( i = 0; i < dataSegVec->length; ++i )
    {


      /*
       *
       * data channel
       *
       */


      /* XXX checks on the length of the input data are missing */

      /* point to current segment */
      DataSegment *dummySegment = dataSegVec->data + i;

      /* endOfData should only be revelant if the data is comming */
      /* in from GetData() for wrapperAPI we know how much data   */
      /* we should be getting                                     */
      dummySegment->endOfData = 0;
      /* this should be set to a unique number for each segment   */
      dummySegment->number = i;

      /* copy the ifodmro */
      memcpy( dummySegment->real4Data->data->data, dummyData, 
          params->initParams->numPoints*sizeof(REAL4) ); 
      dummySegment->real4Data->data->length = params->initParams->numPoints;
      dummyData += (params->initParams->numPoints - params->ovrlap);

      strncpy( dummySegment->real4Data->name, INPUTNAME_CHANNEL, 
          LALNameLength * sizeof(CHAR) );

      dummySegment->real4Data->deltaT = 
        inout->sequences[dataIndex].range.dTime.timeStepSize;

      {
        INT8 dummyNS = 0;

        dummyNS = dataTimeNS + (INT8) (1e9 * 
            (params->initParams->numPoints - params->ovrlap) 
            * i * dummySegment->real4Data->deltaT);

        dummySegment->real4Data->epoch.gpsSeconds = 
          (INT4) (dummyNS/1000000000L);

        dummySegment->real4Data->epoch.gpsNanoSeconds = 
          (INT4) (dummyNS%1000000000L);
      }

      dummySegment->real4Data->f0 = 0.0;

      /* XXX need to fix this to take account of John's new units */
      /* sprintf( dummySegment->data->sampleUnits->data, 
         inout->sequences[dataIndex].units ); */


      /* 
       *
       * determine whether or not we have a one or two sided PSD 
       *
       */


#ifdef INSPIRAL_SO_DEBUG
      fprintf( stdout, "startFreq = %f\n", 
          inout->sequences[specIndex].range.dFreq.startFreq );
      fprintf( stdout, "numberSamples = %Lu\n",
          inout->sequences[specIndex].range.dFreq.numberSamples );
      fprintf( stdout, "numPoints = %d\n",
          params->initParams->numPoints );
#endif

      /* if the start frequency is negative it _could_ be two sided... */
      if ( inout->sequences[specIndex].range.dFreq.startFreq < 0 )
      {
        /* ...if the spectrum has numPoints samples... */
        if ( inout->sequences[specIndex].range.dFreq.numberSamples
            == params->initParams->numPoints )
        {
          oneSidedPSD = 0;
        }
        else /* ...otherwise abort */
        {
          ABORT( status, CONDITIONDATAH_ESPEC, CONDITIONDATAH_MSGESPEC );
        }
      }
      else /* it _could_ be a one sided psd... */
      {
        /* ...if the spectrum has numPoints / 2 + 1 samples... */
        if ( inout->sequences[specIndex].range.dFreq.numberSamples
            == (params->initParams->numPoints/2 + 1) )
        {
          oneSidedPSD = 1;
        }
        else /* ...otherwise abort */
        {
          ABORT( status, CONDITIONDATAH_ESPEC, CONDITIONDATAH_MSGESPEC );
        }
      }


      /*
       *
       * power spectrum
       *
       */


      if ( oneSidedPSD )
      {
        /* copy the one sided spectrum */
        memcpy( dummySegment->spec->data->data, dummySpec, 
            (params->initParams->numPoints/2 + 1)*sizeof(REAL4) ); 
      }
      else
      {
        /* copy the two sided spectrum */
        for ( j = 0; j < params->initParams->numPoints/2 ; ++j )
        {
          dummySegment->spec->data->data[j] = 
            2.0 * dummySpec[j + params->initParams->numPoints/2];
        }
        dummySegment->spec->data->data[params->initParams->numPoints/2] 
          = 2.0 * dummySpec[0];
      }
      
      dummySegment->spec->data->length = 
        (params->initParams->numPoints/2 + 1);

      strncpy( dummySegment->spec->name, INPUTNAME_SPECTRUM, 
          LALNameLength * sizeof(CHAR) );

      dummySegment->spec->deltaF = 
        inout->sequences[specIndex].range.dFreq.freqStepSize;

      dummySegment->spec->epoch.gpsSeconds = 
        ( inout->sequences[specIndex].range.dTime.startSec );

      dummySegment->spec->epoch.gpsNanoSeconds = 
        ( inout->sequences[specIndex].range.dTime.startNan );

      dummySegment->spec->f0 = 0.0;

      /* XXX need to fix this to take account of John's new units */
      /* sprintf( dummySegment->spec->sampleUnits->data, 
         inout->sequences[specIndex].units ); */


      /*
       *
       * response function
       *
       */


      /* copy the response */
      memcpy( dummySegment->resp->data->data, dummyResp, 
          (params->initParams->numPoints/2 +1)*sizeof(COMPLEX8) ); 

      dummySegment->resp->data->length=(params->initParams->numPoints/2 + 1);

      strncpy( dummySegment->resp->name, INPUTNAME_RESPONSE, 
          LALNameLength * sizeof(CHAR) );

      dummySegment->resp->deltaF = 
        inout->sequences[respIndex].range.dFreq.freqStepSize;

      dummySegment->resp->epoch.gpsSeconds = 
        ( inout->sequences[respIndex].range.dTime.startSec );

      dummySegment->resp->epoch.gpsNanoSeconds = 
        ( inout->sequences[respIndex].range.dTime.startNan );

      dummySegment->resp->f0 = 0.0;

      /* XXX need to fix this to take account of John's new units */
      /* sprintf( dummySegment->resp->sampleUnits->data, 
         inout->sequences[respIndex].units ); */
    }


    /*
     *
     * condition the data segments
     *
     */


    params->dataParams->deltaT =
      inout->sequences[dataIndex].range.dTime.timeStepSize;

    LALFindChirpSPData( status->statusPtr, params->fcSegVec, dataSegVec, 
        params->dataParams );
    CHECKSTATUSPTR( status );


    /*
     *
     * free memory for data segments
     *
     */


    LALDestroyDataSegmentVector( status->statusPtr, &dataSegVec );
    CHECKSTATUSPTR( status );


    /*
     *
     * set sampling rate in filter and template generation parameters
     *
     */


    params->tmpltParams->deltaT = 
      inout->sequences[dataIndex].range.dTime.timeStepSize;
    params->filterParams->deltaT = 
      inout->sequences[dataIndex].range.dTime.timeStepSize;
    
  }


  /* normal exit */
  DETATCHSTATUSPTR( status );
  RETURN( status );
}


/*
 * 
 * debugging functions 
 *
 */


#ifdef INSPIRAL_SO_DEBUG
static
void
PrintInspiralBank (
    LALStatus           *status,
    InspiralTemplate    *head,
    FILE                *fp
    )
{
  InspiralTemplate    *current;

  INITSTATUS( status, "PrintInspiralBank", CONDITIONDATAC );
  ATTATCHSTATUSPTR( status );

  ASSERT( head, status, CONDITIONDATAH_ENULL, CONDITIONDATAH_MSGENULL );


  /*
   *
   * print the inspiral template parameter bank
   *
   */


  for ( current = head; current; current = current->next)
  {
    fprintf( fp, 
        "%p\t%d\t%f\t%f\t%f\t%f\t%f\t%p\t%p\n",
        current, current->number, 
        current->mass1, current->mass2, current->totalMass,
        current->eta, current->mu,
        current->next, current->fine );
    fflush( fp );
    if ( current->fine )
    {
      PrintInspiralBank( status->statusPtr, current->fine, fp );
      CHECKSTATUSPTR( status );
    }
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

static
void
PrintTemplateNodeList(
    LALStatus                  *status,
    InspiralTemplateNode       *tmpltHead,
    FILE                       *fp
    )
{
  InspiralTemplateNode *current = NULL;

  INITSTATUS( status, "PrintTemplateNodeList", CONDITIONDATAC);

  ASSERT( tmpltHead, status, CONDITIONDATAH_ENULL, CONDITIONDATAH_MSGENULL );

  for ( current = tmpltHead; current; current = current->next )
  {
    fprintf( fp, 
        "----------------------------------------------------------------------\n" );
    fprintf( fp, "Template Node at %p points to template at %p\n",
        current, current->tmpltPtr );
    fflush( fp );
    fprintf( fp, "   number %d: m1 = %f, m2 = %f\n",
        current->tmpltPtr->number, 
        current->tmpltPtr->mass1, 
        current->tmpltPtr->mass2 );
    fflush( fp );
    fprintf( fp, "next node: %p, prev node: %p\n",
        current->next, current->prev );
    fflush( fp );
  }

  RETURN( status );
}
#endif
