/*----------------------------------------------------------------------- 
 * 
 * File Name: InitSearch.c
 *
 * Author: Brown, D. A.
 * 
 * Revision: $Id: InitSearch.c,v 1.2 2001/06/04 05:21:24 duncan Exp $
 * 
 *-----------------------------------------------------------------------
 */

#include "InitSearch.h"

NRCSID( INITSEARCHC, PKG "$Id: InitSearch.c,v 1.2 2001/06/04 05:21:24 duncan Exp $" );

void
LALInitSearch(
    LALStatus            *status,
    void                **searchParams,
    LALInitSearchParams  *initSearchParams
    )
{

  SimpleSearchParams    *params = NULL;

  /* initialize the lal status structure */
  INITSTATUS( status, "LALInitSearch", INITSEARCHC );
  ATTATCHSTATUSPTR( status );


  /*
   *
   * check input, output and parameters
   *
   */


  /* these structures should always be correctly created by the wrapperAPI  */
  /* so we can use the ASSERT() macro which is disabled in production code  */
  /* (when lal is compiled with --disable-debug)                            */

  ASSERT(     searchParams, status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL );
  ASSERT(  ! *searchParams, status, INITSEARCHH_ENNUL, INITSEARCHH_MSGENNUL );
  ASSERT( initSearchParams, status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL );
  
  /* the command line parameters can easily be entered incorrectly at       */
  /* runtime so we should use if() ABORT() test to ensure that errors       */
  /* are always caught even in production code                              */ 

  /* check that there are the correct number of arguments */
  if ( initSearchParams-> argc != 3 )
  {
    ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS ); 
  }

  /* check that the first argument is -filterparams */
  if ( strcmp( initSearchParams->argv[0], "-filterparams" ) )
  { 
    ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS ); 
  }


  /*
   *
   * sanity checks on command line arguments
   *
   */


  /* check that the number of points is greater than zero */
  if ( atoi( initSearchParams->argv[1] ) <= 0 )
  {
    ABORT( status, INITSEARCHH_ENUMZ, INITSEARCHH_MSGENUMZ ); 
  }

  /* check that the number to count up to is non-negative */
  if ( atoi( initSearchParams->argv[2] ) < 0 )
  {
    ABORT( status, INITSEARCHH_ECNTZ, INITSEARCHH_MSGECNTZ ); 
  }


  /*
   *
   * allocate memory for the trivial search parameter structure
   *
   */


  /* this is the structure that carries memory information between the */
  /* four functions in the shared object library and between sucessive */
  /* calls to LALApplySearch()                                         */
  *searchParams = params = LALCalloc( 1, sizeof(SimpleSearchParams) );
  if ( !params )
  {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }


  /*
   *
   * assign the initial parameter values
   *
   */


  /* number of points */
  params->numPoints = (UINT4) atoi( initSearchParams->argv[1] );

  /* number to count up to */
  params->maxCount  = (UINT4) atoi( initSearchParams->argv[2] );

  /* make sure initial count value is zero */
  params->counter = 0;


  /*
   *
   * allocate memory used by shared object
   *
   */


  /* create a vectors to store the conditioned data which we will be */
  /* created in LALConditionData() used in LALApplySearch()          */ 
  LALCreateVector( status->statusPtr, &(params->dataVec), 
      params->numPoints );
  BEGINFAIL( status )
  {
    /* if the call to LALCreateVector() failed, free the memory */
    /* allocated so far before returning control to lalwrapper  */
    CHECKSTATUSPTR( status );
    LALFree( params );
  }
  ENDFAIL( status );


  /* normal exit */
  DETATCHSTATUSPTR( status );
  RETURN( status );
}
