/********************************** <lalVerbatim file="TFCConditionDataCV">
Author: Julien Sylvestre
$Id: ConditionData.c,v 1.5 2001/08/30 18:37:56 julien Exp $
**************************************************** </lalVerbatim> */

#include <lal/LALStdlib.h>
#include <lal/TFClusters.h>
#include <lal/TFCThresholds.h>
#include <lal/AVFactories.h>
#include <ConditionData.h>
#include <InitSearch.h>
#include <strings.h>

NRCSID( CONDITIONDATAC , "power $Id: ConditionData.c,v 1.5 2001/08/30 18:37:56 julien Exp $" );

void
LALConditionData(
    LALStatus             *status,
    LALSearchInput        *inout,
    void                  *searchParams,
    LALMPIParams          *mpiParams
                )
{
  TFCSearchParams *params;

  INITSTATUS (status, "LALConditionData", CONDITIONDATAC);
  ATTATCHSTATUSPTR (status);

  params = (TFCSearchParams *) searchParams;

  /*
   *
   * do not condition the data on the search master
   *
   */

  if ( params->searchMaster )
  {
    /* return immediately */
    DETATCHSTATUSPTR (status);
    RETURN (status);
  }
  else
  {
    RealFFTPlan *plan = NULL;
    UINT4 i, j, nPts, nTSteps;
    COMPLEX8Vector *fTr = NULL;
    REAL4Vector iVec;

    REAL4 norm;
    REAL4 *dmro = NULL, *rho = NULL;
    RiceThresholdParams rtp;

    ASSERT (searchParams, status, CONDITIONDATA_ENULL, CONDITIONDATA_MSGENULL);

    /*
     *
     * grab first element in multiDim data
     *
     */

    ASSERT ( inout->numberSequences == 1, status, CONDITIONDATA_EINPUT, CONDITIONDATA_MSGEINPUT); 

    if(inout->sequences[0].dimensions[0] != params->NData) {
      ABORT (status, CONDITIONDATA_ENDAT, CONDITIONDATA_MSGENDAT);
    }


    /*
    dmro   = inout->sequences[0].data.real4;
    */

    dmro = (REAL4 *)LALMalloc(params->NData * sizeof(REAL4));    
    if(!dmro) {
      ABORT ( status, CONDITIONDATA_EALOC, CONDITIONDATA_MSGEALOC);
    }

    /* cast */
    switch(inout->sequences[0].type) {
      
#define TFCCAST(TYPE__,PTR__) \
    case TYPE__ : \
      for(i=0; i<params->NData; i++) \
	dmro[i] = (REAL4)inout->sequences[0].data.PTR__[i]; \
      break

    TFCCAST(boolean_lu,boolean);
    TFCCAST(char_s,chars);
    TFCCAST(char_u,charu);
    TFCCAST(int_2s,int2s);
    TFCCAST(int_2u,int2u);
    TFCCAST(int_4s,int4s);
    TFCCAST(int_4u,int4u);
    TFCCAST(int_8s,int8s);
    TFCCAST(int_8u,int8u);
    TFCCAST(real_4,real4);
    TFCCAST(real_8,real8);

#undef TFCCAST

    default:
      ABORT ( status, CONDITIONDATA_EDTYP, CONDITIONDATA_MSGEDTYP );
    }

    /* get time info */
    params->tseries->epoch.gpsSeconds = inout->sequences[0].range.dTime.startSec;
    params->tseries->epoch.gpsNanoSeconds = inout->sequences[0].range.dTime.startNan;


    /* compute thresholds; redundant, done on every slave */
    if(params->eGoal > 0.0) {

      rtp.bpp = params->bpp;
      rtp.eGoal = params->eGoal;
      rtp.nFreq = (UINT4)ceil(0.5*params->T/params->tseries->deltaT);

      if(!rtp.nFreq) {
	ABORT ( status, CONDITIONDATA_ENUMZ, CONDITIONDATA_MSGENUMZ );
      }

      rtp.meanRe = (REAL4 *)LALMalloc(rtp.nFreq * sizeof(REAL4));
      if(!rtp.meanRe) {ABORT ( status, CONDITIONDATA_ENULL, CONDITIONDATA_MSGENULL);} 
      rtp.meanIm = (REAL4 *)LALMalloc(rtp.nFreq * sizeof(REAL4));
      if(!rtp.meanIm) {ABORT ( status, CONDITIONDATA_ENULL, CONDITIONDATA_MSGENULL);} 
      rtp.varRe = (REAL4 *)LALMalloc(rtp.nFreq * sizeof(REAL4));
      if(!rtp.varRe) {ABORT ( status, CONDITIONDATA_ENULL, CONDITIONDATA_MSGENULL);} 
      rtp.varIm = (REAL4 *)LALMalloc(rtp.nFreq * sizeof(REAL4));
      if(!rtp.varIm) {ABORT ( status, CONDITIONDATA_ENULL, CONDITIONDATA_MSGENULL);} 
      
      bzero(rtp.meanRe, rtp.nFreq * sizeof(REAL4));
      bzero(rtp.meanIm, rtp.nFreq * sizeof(REAL4));
      bzero(rtp.varRe, rtp.nFreq * sizeof(REAL4));
      bzero(rtp.varIm, rtp.nFreq * sizeof(REAL4));

      nPts = params->T / params->tseries->deltaT;
      nTSteps = (UINT4)floor((double)params->NData / (double)nPts);
      
      if(!nTSteps) {
	ABORT ( status, CONDITIONDATA_ENUMZ, CONDITIONDATA_MSGENUMZ );
      }
    
      iVec.length = nPts;
      
      if(!nPts) {
	ABORT ( status, CONDITIONDATA_ENUMZ, CONDITIONDATA_MSGENUMZ );
      }

      /* old function
	 LALEstimateForwardRealFFTPlan(status->statusPtr, &plan, nPts);
      */

      LALCreateForwardRealFFTPlan(status->statusPtr, &plan, nPts, 0);
      CHECKSTATUSPTR (status);
      
      LALCCreateVector(status->statusPtr, &fTr, nPts/2 + 1);
      CHECKSTATUSPTR (status);

      norm = sqrt((REAL4)nPts);

      for(i=0;i<nTSteps;i++) {
	iVec.data = dmro + i * nPts;
	LALForwardRealFFT(status->statusPtr, fTr, &iVec, plan);

	for(j=1; j<nPts/2+1; j++) {
	  fTr->data[j].re /= norm;
	  fTr->data[j].im /= norm;

	  /* NOTE: rho will be computed from 1/T to Nyquist, i.e. not for DC */
	  rtp.meanRe[j-1] += fTr->data[j].re;
	  rtp.meanIm[j-1] += fTr->data[j].im;

	  rtp.varRe[j-1] += fTr->data[j].re * fTr->data[j].re;
	  rtp.varIm[j-1] += fTr->data[j].im * fTr->data[j].im;
	}
      }

      for(j=0; j<nPts/2; j++) {
	rtp.meanRe[j] /= (REAL4)nTSteps;
	rtp.meanIm[j] /= (REAL4)nTSteps;
	
	rtp.varRe[j] = rtp.varRe[j] / (REAL4)(nTSteps - 1) - (REAL4)nTSteps * rtp.meanRe[j] * rtp.meanRe[j] / (REAL4)(nTSteps - 1);
	rtp.varIm[j] = rtp.varIm[j] / (REAL4)(nTSteps - 1) - (REAL4)nTSteps * rtp.meanIm[j] * rtp.meanIm[j] / (REAL4)(nTSteps - 1);
      }

      rho = (REAL4 *)LALMalloc(rtp.nFreq * sizeof(REAL4));
      if(!rho) {ABORT ( status, CONDITIONDATA_ENULL, CONDITIONDATA_MSGENULL);}
      
      LALTFCRiceThreshold(status->statusPtr, rho, &rtp);
      CHECKSTATUSPTR (status);
      
      for(j=0;j<rtp.nFreq;j++) {
	params->dir->rho[j] = (REAL8)rho[j];
      }

      params->dir->freqBins = rtp.nFreq;

      /* clean up */

      LALCDestroyVector(status->statusPtr, &fTr);
      CHECKSTATUSPTR (status);

      LALDestroyRealFFTPlan(status->statusPtr, &plan);
      CHECKSTATUSPTR (status);

      LALFree(rho);

      LALFree(rtp.meanRe);
      LALFree(rtp.meanIm);
      LALFree(rtp.varRe);
      LALFree(rtp.varIm);
    }
    else { /* no need to compute threshold: white noise */

      UINT4 nFreq = (UINT4)ceil(0.5*params->T/params->tseries->deltaT);
      REAL8 thr = -log(params->bpp);

      if(!nFreq || isnan(thr)) {
	ABORT ( status, CONDITIONDATA_ENUMZ, CONDITIONDATA_MSGENUMZ );
      }


      for(j=0;j<nFreq;j++) {
	params->dir->rho[j] = thr;
      }

      params->dir->freqBins = nFreq;
    }


    params->dmro = (REAL4*)dmro;    
  } /* else of if(serachMaster) */


  DETATCHSTATUSPTR (status);
  RETURN (status);
}

