/********************************** <lalVerbatim file="TFCInitSearchCV">
Author: Julien Sylvestre
$Id: InitSearch.c,v 1.4 2001/08/28 17:12:45 julien Exp $
**************************************************** </lalVerbatim> */

#include <lal/LALStdlib.h>
#include <lal/TFClusters.h>
#include <lal/AVFactories.h>
#include <InitSearch.h>
#include <unistd.h>

NRCSID( INITSEARCHC, "tfclusters $Id: InitSearch.c,v 1.4 2001/08/28 17:12:45 julien Exp $");


/* PARAMETERS:
   argv[0] = "-filterparams"
   [1] = total number of data
   [2] = number of samples in one segment
   [3] = time resolution in seconds
   [4] = sampling frequency 
   [5] = overlap between segments (number of samples)
   [6] = black pixel probability
   [7] = absolute error goal on power threshld (Set to <0 for white noise of unit variance at input)
   [8] = alpha
   [9] = minimum frequency to process 
   [10] = maximum frequency to process
   [11] = sigma
   [12] = distance for s1 = 1, s2 = 1
   [13] = distance for s1 = 1, s2 = 2
   ...
   [.] = distance for s1 = 1, s2 = sigma - 1
   [..] = distance for s1 = 2, s2 = 2
   ...
   [] = distance for s1 = sigma-1, s2 = sigma-2
*/

void LALInitSearch(
    LALStatus             *status,
    void                 **searchParams,
    LALInitSearchParams   *initSearchParams
    )
{

  TFCSearchParams *params;
  REAL8 T, alpha=0, minf, maxf;
  INT4 sigma, tmpi4;
  UINT4 i, olap, delL;
  INT4 fs, Ttotal, NData;
  UINT4 *delta;
  REAL4Vector *vect;
  REAL4 bpp, eGoal;


  INITSTATUS (status, "LALInitSearch", INITSEARCHC);
  ATTATCHSTATUSPTR (status);


  /*
   *
   * Sanity checks on the input, output, params
   *
   */

  if ( !(initSearchParams) ){
    ABORT( status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL);
  }
  if ( (*searchParams) ){
    ABORT( status, INITSEARCHH_ENNUL, INITSEARCHH_MSGENNUL);
  }

  if ( !(initSearchParams->argv) ){
    ABORT( status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL);
  }

  if ( initSearchParams->argc < 11){
        ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS ); 
  }
 
  if (strcmp( initSearchParams->argv[0], "-filterparams" )){
    ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS);
  }



  NData = atoi(initSearchParams->argv[1]);
  if ( NData <= 0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }


  Ttotal = atoi(initSearchParams->argv[2]);
  if ( Ttotal <= 0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }

  T = atof(initSearchParams->argv[3]);
  if ( isnan(T) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( T <= 0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }

  fs = atoi(initSearchParams->argv[4]);
  if ( fs <= 0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }


  olap = (unsigned int)atoi(initSearchParams->argv[5]);
  if ( olap < 0 ) {
    ABORT ( status, INITSEARCHH_EPOS, INITSEARCHH_MSGEPOS );
  }

  bpp = atof(initSearchParams->argv[6]);
  if ( isnan(bpp) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( bpp < 0.0 || bpp > 1.0 ) {
    ABORT ( status, INITSEARCHH_EAOR, INITSEARCHH_MSGEAOR );
  }

  eGoal = atof(initSearchParams->argv[7]);
  if ( isnan(eGoal) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }

  alpha = atof(initSearchParams->argv[8]);
  if ( isnan(alpha) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( alpha < 0.0 || alpha > 1.0 ) {
    ABORT ( status, INITSEARCHH_EAOR, INITSEARCHH_MSGEAOR );
  }

  minf = atof(initSearchParams->argv[9]);
  if ( isnan(minf) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( minf < 0.0 ) {
    ABORT ( status, INITSEARCHH_EPOS, INITSEARCHH_MSGEPOS );
  }

  maxf = atof(initSearchParams->argv[10]);
  if ( isnan(maxf) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( maxf < 0.0 ) {
    ABORT ( status, INITSEARCHH_EPOS, INITSEARCHH_MSGEPOS );
  }

  if( minf > maxf ) {
    ABORT ( status, INITSEARCHH_EMMF, INITSEARCHH_MSGEMMF );
  }
  
  sigma = atoi(initSearchParams->argv[11]);
  if(sigma<=0) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }

  delL = sigma * (sigma - 1) / 2;

  if(initSearchParams->argc != 12 + delL) {
    ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS );
  }

  if(sigma > 1) {

    delta = (UINT4 *) LALMalloc ( delL * sizeof(UINT4) );
    if(!delta) {
      ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
    }

    for(i=0; i<delL; i++) {
    
      tmpi4 = atoi(initSearchParams->argv[12+i]);
      if(tmpi4 < 0) {
	ABORT ( status, INITSEARCHH_EPOS, INITSEARCHH_MSGEPOS );
      }

      delta[i] = (UINT4)tmpi4;
    }
  }
  else delta = NULL;


  /*
   *
   * allocate memory 
   *
   */

  *searchParams = params = LALMalloc (sizeof( TFCSearchParams )); 
  if ( !params )
  {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }


  params->dir = (CListDir*)LALMalloc(sizeof(CListDir));
  if(!params->dir) {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }

  params->tseries = (REAL4TimeSeries*) LALMalloc (sizeof(REAL4TimeSeries));
  if(!params->tseries) {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }

  params->tspec = (TFPlaneParams*) LALMalloc (sizeof(TFPlaneParams));
  if(!params->tspec) {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }


  vect = NULL;

  params->tseries->deltaT=1.0/(REAL8)fs;
  params->tseries->f0 = 0.0;
  params->tseries->data = vect;

  params->dir->freqBins = (UINT4)(fs*T/2.0);
  params->dir->sigma = (UINT4)sigma;
  params->dir->minf = minf;
  params->dir->maxf = maxf;
  params->dir->alpha = alpha;

  LALFillCListDir(status->statusPtr, params->dir, 1);
  CHECKSTATUSPTR (status);

  if(delta) {
    memcpy(params->dir->d, delta, delL * sizeof(UINT4));
  
    params->dir->mdist = delta[0];
    for(i=1; i<delL; i++)
      if(delta[i] > params->dir->mdist) {params->dir->mdist = delta[i];}

    LALFree(delta);
  }
  else params->dir->mdist = 0;

  /*
   * 
   * assign the initial parameter values
   *
   */
  
  params->NData = (UINT4)NData;
  params->T = T;
  params->Ttotal = (UINT4)Ttotal;
  params->olap = olap;
  params->bpp = bpp;
  params->eGoal = eGoal;

  params->searchMaster = initSearchParams->nodeClass;

  params->dmro = NULL;

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

